/**
 * 工具类服务
 */
angular
	.module("kyee.framework.service.utils", [])
	.factory("KyeeUtilsService", ["$timeout", "$interval", "$sce", function($timeout, $interval, $sce){
		
		return {

			/**
			 * 延迟执行函数
			 *
			 * @param cfg{
			 * 	time : 延迟时间（单位 ms）,
			 * 	do : 延迟执行函数
			 * }
			 */
			delay : function(cfg){
				
				return $timeout(cfg.action, cfg.time);
			},

			/**
			 * 取消延迟执行器
			 *
			 * @param timer
			 */
			cancelDelay : function(timer){

				$timeout.cancel(timer);
			},

			/**
			 * 循环执行函数
			 *
			 * @param cfg
			 */
			interval : function(cfg){

				return $interval(cfg.action, cfg.time);
			},

			/**
			 * 取消循环定时器
			 *
			 * @param timer
			 */
			cancelInterval : function(timer){

				$interval.cancel(timer);
			},

			/**
			 * 获取浏览器可视区域大小
			 *
			 * @returns {{width: Number, height: Number}}
			 */
			getInnerSize : function(){

				return {
					width : window.innerWidth,
					height : window.innerHeight
				}
			},

			/**
			 * 检查 JSON 是否包含语法错误
			 *
			 * @param jsonString
			 * @returns {boolean}
			 */
			isJSONError : function(jsonString){

				var checkResult = false;

				if(jsonString == undefined || jsonString == null || jsonString == ""){
					return checkResult;
				}

				try{

					//不要使用 var 声明，以免变量重复
					eval("kyee_json_test_result="+jsonString+";");

					checkResult = false;
				}catch(e){

					checkResult = true;
				}finally{

					return checkResult;
				}
			},

			/**
			 * 获取日期工具
			 *
			 * @returns {{}}
			 */
			DateUtils : {

				/**
				 * 获取当前日期
				 *
				 * @param pattern 格式化参数，例如：YYYY-MM-DD，如果不指定该参数，默认为 YYYY/MM/DD
				 * @returns {*}
				 */
				getDate : function(pattern){

					if(pattern == undefined){
						pattern = "YYYY/MM/DD";
					}

					return moment().format(pattern);
				},

				/**
				 * 获取当前时间
				 *
				 * @param pattern 格式化参数，例如：HH:mm:ss，如果不指定该参数，默认为 HH:mm（注意：HH 为 24 小时制，hh 为 12 小时制）
				 * @returns {*}
				 */
				getTime : function(pattern){

					if(pattern == undefined){
						pattern = "HH:mm";
					}

					return moment().format(pattern);
				},

				/**
				 * 获取当前日期时间
				 *
				 * @param pattern 格式化参数，例如：YYYY-MM-DD HH:mm:ss，如果不指定该参数，默认为 YYYY/MM/DD HH:mm（注意：HH 为 24 小时制，hh 为 12 小时制）
				 * @returns {*}
				 */
				getNow : function(pattern){

					if(pattern == undefined){
						pattern = "YYYY/MM/DD HH:mm";
					}

					return moment().format(pattern);
				},

				/**
				 * 日期格式化
				 *
				 * @param src 源日期字符串
				 * @param srcPattern 源日期格式（注意：HH 为 24 小时制，hh 为 12 小时制）
				 * @param targetPattern 目标日期格式（注意：HH 为 24 小时制，hh 为 12 小时制）
				 * @returns {*}
				 */
				formatFromString : function(src, srcPattern, targetPattern){

					return moment(src, srcPattern).format(targetPattern);
				},

				/**
				 * 日期格式化
				 *
				 * @param srcDate 源日期对象
				 * @param pattern 格式化参数（注意：HH 为 24 小时制，hh 为 12 小时制）
				 * @returns {*}
				 */
				formatFromDate : function(srcDate, pattern){

					return moment(srcDate).format(pattern);
				},

				/**
				 * 日期解析
				 *
				 * @param src 日期字符串
				 * @param pattern 格式化参数（注意：HH 为 24 小时制，hh 为 12 小时制）
				 * @returns {*}
				 */
				parse : function(src, pattern){

					return moment(src, pattern).toDate();
				}
			},

			/**
			 * 获取 url
			 * <br/>
			 * 此操作会对网址做安全检查
			 *
			 * @param url
			 * @returns {*}
			 */
			getUrl : function(url){

				return $sce.trustAsResourceUrl(url);
			},

			/**
			 * 安全类工具服务
			 */
			SecurityUtils : {

				/**
				 * md5 摘要算法
				 *
				 * @param str
				 * @returns {*}
				 */
				md5 : function(str){

					return MD5.encode(str);
				}
			},

			/**
			 * 构造 url
			 *
			 * @param url
			 * @param params
			 * @returns {*}
			 */
			buildUrl : function(url, params){

				var me = this;

				if (!params) return url;
				var parts = [];
				me._forEachSorted(params, function(value, key) {
					if (value === null || angular.isUndefined(value)) return;
					if (!angular.isArray(value)) value = [value];

					angular.forEach(value, function(v) {
						if (angular.isObject(v)) {
							if (angular.isDate(v)) {
								v = v.toISOString();
							} else {
								v = angular.toJson(v);
							}
						}
						parts.push(me._encodeUriQuery(key) + '=' +
							me._encodeUriQuery(v));
					});
				});
				if (parts.length > 0) {
					url += ((url.indexOf('?') == -1) ? '?' : '&') + parts.join('&');
				}
				return url;
			},

			_forEachSorted : function(obj, iterator, context){

				var me = this;

				var keys = me._sortedKeys(obj);
				for (var i = 0; i < keys.length; i++) {
					iterator.call(context, obj[keys[i]], keys[i]);
				}
				return keys;
			},

			_sortedKeys : function(obj) {
				return Object.keys(obj).sort();
			},

			_encodeUriQuery : function(val, pctEncodeSpaces) {
				return encodeURIComponent(val).
					replace(/%40/gi, '@').
					replace(/%3A/gi, ':').
					replace(/%24/g, '$').
					replace(/%2C/gi, ',').
					replace(/%3B/gi, ';').
					replace(/%20/g, (pctEncodeSpaces ? '%20' : '+'));
			},

			/**
			 * 退出应用程序
			 */
			exitApp : function(){

				if(ionic.Platform.platform() == "ios"){
					//edit by zhangwenhua HRPQD-334
					//ios需调用自定义插件结束进程
					if(navigator.IMManager != undefined && navigator.IMManager != null){
						navigator.IMManager.callIMFunction(null,null, [{
							'function': 'exitApp',
							'params': ''
						}]);
					}
				}else{
                    if(navigator.IMManager != undefined && navigator.IMManager != null){
                        navigator.IMManager.callIMFunction(null,null, [{
                            'function': 'exitApp',
                            'params': {}
                        }]);
                    }
				}
			}
		};
}]);